/* VERSION determines the program's version number
 * This file is part of jdupes; see jdupes.c for license information */

#ifndef JDUPES_VERSION_H
#define JDUPES_VERSION_H

#define VER "1.20.2"
#define VERDATE "2021-11-02"

#endif /* JDUPES_VERSION_H */
